// ShowSysInfo.cpp
//
// Displays information contained within the SYSTEM_INFO structure.
//
#include <windows.h>
#include <iostream>
#include <string>

int main(int argc, char* argv[])
{
	// Retrieve data
	SYSTEM_INFO si = {0};
	GetSystemInfo(&si);

	// Display results
	std::cout << "System Information" << std::endl;
	std::cout << "==================" << std::endl;

	// Architecture
	std::string strPArch("  Architecture: ");
	switch ( si.wProcessorArchitecture ) {
		case PROCESSOR_ARCHITECTURE_INTEL:
			strPArch += "Intel";
			switch ( si.wProcessorLevel ) {
				case 3:
					strPArch += " (386)";
					break;

				case 4:
					strPArch += " (486)";
					break;

				case 5:
					strPArch += " (Pentium)";
					break;

				case 6:
					strPArch += " (Pentium Pro, II, or III)";
					break;

				default:
					strPArch += " (unknown type)";
					break;

			} // switch
			break;

		case PROCESSOR_ARCHITECTURE_MIPS:
			strPArch += "MIPS";
			switch ( si.wProcessorLevel ) {
				case 4:
					strPArch += " (R4000)";
					break;

				default:
					strPArch += " (unknown type)";
					break;

			} // switch
			break;

		case PROCESSOR_ARCHITECTURE_ALPHA:
			strPArch += "DEC Alpha";
			switch ( si.wProcessorLevel ) {
				case 21064:
					strPArch += " (21064)";
					break;

				case 21066:
					strPArch += " (21066)";
					break;

				case 21164:
					strPArch += " (21164)";
					break;

				default:
					strPArch += " (unknown type)";
					break;

			} // switch
			break;

		case PROCESSOR_ARCHITECTURE_PPC:
			strPArch += "PowerPC";
			switch ( si.wProcessorLevel ) {
				case 1:
					strPArch += " (601)";
					break;

				case 3:
					strPArch += " (603)";
					break;

				case 4:
					strPArch += " (604)";
					break;

				case 6:
					strPArch += " (603+)";
					break;

				case 9:
					strPArch += " (604+)";
					break;

				case 20:
					strPArch += " (620)";
					break;

				default:
					strPArch += " (unknown type)";
					break;

			} // switch
			break;

		case PROCESSOR_ARCHITECTURE_UNKNOWN:
		default:
			strPArch += "Unknown";
			break;

	} // switch

	std::cout << strPArch.c_str() << std::endl;

	// Number of processors
	std::cout << "  Number of processors: " << si.dwNumberOfProcessors << std::endl;

	// Processor active mask
	if ( si.dwNumberOfProcessors > 1 ) {
		// Spit out processor active mask setting per processor
		for ( int i = 0; i < si.dwNumberOfProcessors; i++ ) {
			// This processor...
			std::cout << "    Processor " << i << ": ";
			if ( (si.dwActiveProcessorMask >> i) & 0x00000001 ) {
				// Active
				std::cout << "Active" << std::endl;
			} // if
			else {
				// Inactive
				std::cout << "Inactive" << std::endl;
			} // else
		} // for
	} // if

	return 0;
}
